@@ -196,7 +196,6 @@ int ffsrow( fitsfile *infptr,   /* I - Input FITS file                      */
       return( *status = PARSE_BAD_TYPE );
    }
 
-
    /***********************************************************/
    /*  Extract various table information from each extension  */
    /***********************************************************/
@@ -269,12 +268,12 @@ int ffsrow( fitsfile *infptr,   /* I - Input FITS file                      */
       /* Error... Do nothing */
    } else {
       rdlen  = inExt.rowLength;
-      buffer = (unsigned char *)malloc( maxvalue(100000,rdlen) * sizeof(char) );
+      buffer = (unsigned char *)malloc(maxvalue(500000,rdlen) * sizeof(char) );
       if( buffer==NULL ) {
          ffcprs();
          return( *status=MEMORY_ALLOCATION );
       }
-      maxrows = 100000L/rdlen;
+      maxrows = 500000L/rdlen;
       nbuff = 0;
       inloc = 1;
       if( infptr==outfptr ) { /* Skip initial good rows if input==output file */
@@ -282,7 +281,8 @@ int ffsrow( fitsfile *infptr,   /* I - Input FITS file                      */
 	 outloc = inloc;
       } else {
 	 outloc = outExt.numRows + 1;
-	 ffirow( outfptr, outExt.numRows, nGood, status );
+	 if (outloc > 1) 
+            ffirow( outfptr, outExt.numRows, nGood, status );
       }
 
       do {
@@ -354,7 +354,7 @@ int ffsrow( fitsfile *infptr,   /* I - Input FITS file                      */
          outbyteloc = outExt.heapStart + outExt.dataStart + outExt.heapSize;
 
          while ( ntodo && !*status ) {
-            rdlen = minvalue(ntodo,100000);
+            rdlen = minvalue(ntodo,500000);
             ffmbyt( infptr,  inbyteloc,  REPORT_EOF, status );
             ffgbyt( infptr,  rdlen,  buffer,     status );
             ffmbyt( outfptr, outbyteloc, IGNORE_EOF, status );
@@ -390,6 +390,8 @@ int ffsrow( fitsfile *infptr,   /* I - Input FITS file                      */
 
    free(Info.dataPtr);
    ffcprs();
+
+   ffcmph(outfptr, status);  /* compress heap, deleting any orphaned data */
    return(*status);
 }
 
@@ -654,6 +656,9 @@ int ffcalc_rng( fitsfile *infptr,   /* I - Input FITS file                  */
 
       int anyNull = 0;
       int nPerLp, i;
+      long totaln;
+
+      ffgkyj(infptr, "NAXIS2", &totaln, 0, status);
 
       /*************************************/
       /* Create new iterator Output Column */
@@ -672,10 +677,20 @@ int ffcalc_rng( fitsfile *infptr,   /* I - Input FITS file                  */
       for( i=0; i<nRngs; i++ ) {
          Info.dataPtr = NULL;
          Info.maxRows = end[i]-start[i]+1;
-         if( Info.maxRows < 10 )
-            nPerLp = Info.maxRows;
+
+          /*
+            If there is only 1 range, and it includes all the rows,
+            and there are 10 or more rows, then set nPerLp = 0 so
+            that the iterator function will dynamically choose the
+            most efficient number of rows to process in each loop.
+            Otherwise, set nPerLp to the number of rows in this range.
+         */
+
+         if( (Info.maxRows >= 10) && (nRngs == 1) &&
+             (start[0] == 1) && (end[0] == totaln))
+              nPerLp = 0;
          else
-            nPerLp = 0;
+              nPerLp = Info.maxRows;
 
          if( ffiter( gParse.nCols, gParse.colData, start[i]-1,
                      nPerLp, parse_data, (void*)&Info, status ) == -1 )
@@ -2073,10 +2088,12 @@ static int find_column( char *colName, void *itslval )
       if(fits_read_key(fptr,TDOUBLE,temp,&tscale,NULL,&istatus)) {
           tscale = 1.0;
       } 
-      if (tscale == 1.0 && (tzero == 0.0 || tzero == 32768.0 || 
-                            tzero == 2147483648.0 )) {
+      if (tscale == 1.0 && (tzero == 0.0 || tzero == 32768.0 )) {
           gParse.varData[col_cnt].type     = LONG;
           gParse.colData[col_cnt].datatype = TLONG;
+      } else if (tscale == 1.0 &&  tzero == 2147483648.0 ) {
+          gParse.varData[col_cnt].type     = LONG;
+          gParse.colData[col_cnt].datatype = TULONG;
       } else {
           gParse.varData[col_cnt].type     = DOUBLE;
           gParse.colData[col_cnt].datatype = TDOUBLE;
